/*************************************************
*
*     PROJECT: XbpColumn:heading & lbClick & Sort
*        FILE: Test.zip
*  Programmer: J.A. Diego Kerejeta
*     Created: 06/05/2018
*
**************************************************/

#include 'Appevent.ch'
#include 'Common.ch'
#include 'Gra.ch'
#include 'Xbp.ch'
#include 'Font.ch'

#pragma Library( "XppUi2.lib" )

#define DEFAULT_HEADING_POINTER   XBPSTATIC_SYSICON_SIZENS   // IMPORTANT, define a more appropriate pointer

#define xbeP_Fin                  xbeP_User + 1
#define TIT_TESTS                'Tests'
#define TXT_TEST_1               'Test 1'
#define TXT_MENU                 'Menu'
#define TXT_QUIT                 'Quit'
#define MSG_DBE_NOT_LOADED       ' database engine not loaded'
#define MSG_DBE_NOT_CREATED      ' database engine could not be created'

****************
PROCEDURE DbeSys
****************
SET COLLATION TO AMERICAN
SET DATE TO AMERICAN

IF     ! DbeLoad( 'FOXDBE', .T. )                 ; Alert( 'FOXDBE' + MSG_DBE_NOT_LOADED,  { 'OK' } ) ; QUIT
ELSEIF ! DbeLoad( 'CDXDBE', .T. )                 ; Alert( 'CDXDBE' + MSG_DBE_NOT_LOADED,  { 'OK' } ) ; QUIT
ELSEIF ! DbeBuild( 'FOXCDX', 'FOXDBE', 'CDXDBE' ) ; Alert( 'FOXCDX' + MSG_DBE_NOT_CREATED, { 'OK' } ) ; QUIT
ENDIF
DbeSetDefault( 'FOXCDX' )
RETURN

****************
PROCEDURE AppSys
****************
LOCAL oDlg

oDlg := XbpDialog():new( , , { 0, 0 }, { 790, 600 }, , .F. )
   oDlg:taskList                := .T.
   oDlg:close                   := {|u1, u2, obj| PostAppEvent( xbeP_Fin,,, obj ) }
   oDlg:border                  := XBPDLG_DLGBORDER
   oDlg:title                   := TIT_TESTS
   oDlg:drawingArea:clipChildren:= .T.
   oDlg:create()
   AppMenu( oDlg:menuBar() )
   oDlg:setFrameState( XBPDLG_FRAMESTAT_MAXIMIZED )

SetAppFocus( oDlg )
SetAppWindow( oDlg )
RETURN

*****************************
PROCEDURE AppMenu( oMenuBar )
*****************************
LOCAL oMenu

oMenu:= XbpMenu():new( oMenuBar )
   oMenu:title := TXT_MENU
   oMenu:create()
   oMenu:addItem( { TXT_TEST_1,  {|| Test( 1 ) } } )
   oMenu:addItem( { NIL, NIL, XBPMENUBAR_MIS_SEPARATOR, 0} )
   oMenu:addItem( { TXT_QUIT,  {|| PostAppEvent( xbeP_Fin ) } } )
   oMenuBar:addItem( { oMenu, } )
RETURN

**************
PROCEDURE Main
**************
LOCAL nEvent, mp1, mp2, oXbp

DO WHILE nEvent <> xbeP_Fin
   nEvent := AppEvent( @mp1, @mp2, @oXbp )
   oXbp:handleEvent( nEvent, mp1, mp2 )
ENDDO
QUIT
RETURN

***********************
PROCEDURE Test( nTest )
***********************
LOCAL oXbp, mp1, mp2, nEvent, oDlg2, nCont, oBrowse
LOCAL cPath    := StrTran( AppName( .T. ), AppName(), '' )
LOCAL aSizeDesk:= AppDesktop():currentSize()
LOCAL aSize    := { 470, 550 }
LOCAL aPos     := { INT( ( aSizeDesk[ 1 ] - aSize[ 1 ] ) / 2 ), ;
                    INT( ( aSizeDesk[ 2 ] - aSize[ 2 ] ) / 2 ) }
LOCAL aStruct  := { { 'CAMPO_111', 'C', 25, 0  }, ;
                    { 'CAMPO_222', 'N',  6, 0  }, ;
                    { 'CAMPO_333', 'C', 25, 0  }, ;
                    { 'CAMPO_444', 'N',  6, 0  } }

FIELD CAMPO_111, CAMPO_222, CAMPO_333, CAMPO_444

IF ! FExists( 'KK.DBF' )
   DBcreate( 'KK', aStruct )

   USE KK NEW EXCLUSIVE

   FOR nCont:= 1 TO 100
      KK->( DBappend() )
      KK->CAMPO_111:= CMONTH( DATE() + 31 * nCont )
      KK->CAMPO_222:= nCont + 1000
      KK->CAMPO_333:= CMONTH( DATE() + 31 * ( 37 - nCont ) )
      KK->CAMPO_444:= 37 - nCont + 1000
   NEXT
   KK->( DBgoTop() )
ELSE
   USE KK NEW EXCLUSIVE
ENDIF
IF ! FExists( cPath + 'KK.CDX' )
   KK->( OrdCreate( cPath + 'KK',  'I1', 'CAMPO_111', {|| CAMPO_111 } ), ;
         OrdCreate( cPath + 'KK',  'I2', 'CAMPO_222', {|| CAMPO_222 } ), ;
         OrdCreate( cPath + 'KK',  'I3', 'CAMPO_333', {|| CAMPO_333 } ), ;
         OrdCreate( cPath + 'KK',  'I4', 'CAMPO_444', {|| CAMPO_444 } ), ;
         OrdCondSet(,,,,,,,,,,.T. ), ;
         OrdCreate( cPath + 'KK', 'I11', 'CAMPO_111', {|| CAMPO_111 } ), ;
         OrdCondSet(,,,,,,,,,,.T. ), ;
         OrdCreate( cPath + 'KK', 'I12', 'CAMPO_222', {|| CAMPO_222 } ), ;
         OrdCondSet(,,,,,,,,,,.T. ), ;
         OrdCreate( cPath + 'KK', 'I13', 'CAMPO_333', {|| CAMPO_333 } ), ;
         OrdCondSet(,,,,,,,,,,.T. ), ;
         OrdCreate( cPath + 'KK', 'I14', 'CAMPO_444', {|| CAMPO_444 } ) )
ENDIF
KK->( OrdListAdd( cPath + 'KK' ), OrdSetFocus( 0 ) )

oDlg2 := XbpDialog():new( AppDeskTop(), SetAppWindow(), aPos, aSize,, .F. )
   oDlg2:taskList := .T.
   oDlg2:drawingArea:ClipChildren := .T.
   oDlg2:create()
   oDlg2:drawingArea:setFontCompoundName( FONT_DEFPROP_SMALL )
   oDlg2:drawingArea:resize := {|mp1,mp2,obj| obj:childList()[1]:setSize(mp2) }

IF nTest = 1
   oDlg2:setTitle( TXT_TEST_1 )
   oBrowse:= BrowseTest( oDlg2:drawingArea, aStruct )
ENDIF

oDlg2:setModalState( XBP_DISP_APPMODAL )
oBrowse:show()
oDlg2:show()

SetAppFocus( oBrowse )

nEvent:= xbe_None

DO WHILE nEvent <> xbeP_Close
   nEvent := AppEvent( @mp1, @mp2, @oXbp )
   oXbp:handleEvent( nEvent, mp1, mp2 )
ENDDO
oBrowse:destroy()
oDlg2:setModalState( XBP_DISP_MODELESS )
oDlg2:destroy()
KK->( DBcloseArea() )
RETURN

***************************************
FUNCTION BrowseTest( oParent, aStruct )
***************************************
LOCAL aPP, oBrowse, oColumn

oBrowse := XbpBroTest():new( oParent,, { 0, 0 },,, .F. )
   oBrowse:nArea:= Select()
   oBrowse:create()

aPP := { { XBP_PP_COL_HA_CAPTION, aStruct[ 1, 1 ] + Space( 10 ) } }

oColumn := XbpColTest():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_111 }
   oBrowse:addColumn( oColumn )

aPP[ 1, 2 ] := aStruct[ 2 , 1 ] + Space( 10 )

oColumn := XbpColTest():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_222 }
   oBrowse:addColumn( oColumn )

aPP[ 1, 2 ] := aStruct[ 3 , 1 ] + Space( 10 )

oColumn := XbpColTest():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_333 }
   oBrowse:addColumn( oColumn )

aPP[ 1, 2 ] := aStruct[ 4 , 1 ] + Space( 10 )

oColumn := XbpColTest():new( oBrowse,,,, aPP )
   oColumn:dataLink:= {|x| KK->CAMPO_444 }
   oBrowse:addColumn( oColumn )
RETURN oBrowse

*******************************
CLASS XbpBroTest FROM XbpBrowse
*******************************
PROTECTED:
   VAR oColOrder
   VAR lDescend

   INLINE METHOD DrawMarker( oPS, aPos )
   *************************************
      LOCAL nXSize:= 8
      LOCAL nCont

      IF ::lDescend
         FOR nCont:= 0 TO 4
            GraLine( oPS, aPos, { aPos[ 1 ] + nXSize, aPos[ 2 ] } )
            aPos[ 1 ]++ ; aPos[ 2 ]-- ; nXSize -= 2
         NEXT
      ELSE
         aPos[ 2 ]-= 4
         FOR nCont:= 0 TO 4
            GraLine( oPS, aPos, { aPos[ 1 ] + nXSize, aPos[ 2 ] } )
            aPos[ 1 ]++ ; aPos[ 2 ]++ ; nXSize -= 2
         NEXT
      ENDIF
   RETURN self

EXPORTED:
   VAR nArea

   INLINE METHOD Destroy
   *********************
      ::XbpBrowse:destroy()
      ::oColOrder:= ::lDescend:= ::nArea:= nil
   RETURN self

   INLINE METHOD Init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *****************************************************************
      ::XbpBrowse:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
         ::skipBlock     := {|n| ( ::nArea )->( DbSkipper(n) ) }   // Use MyDbfSkipper( n, ::nArea )
         ::goTopBlock    := {| | ( ::nArea )->( DbGoTop()    ) }
         ::goBottomBlock := {| | ( ::nArea )->( DbGoBottom() ) }
         ::phyPosBlock   := {| | ( ::nArea )->( Recno()      ) }
         ::posBlock      := {| | ( ::nArea )->( DbPosition() ) }
         ::goPosBlock    := {|n| ( ::nArea )->( DbGoPosition(n) ) }
         ::lastPosBlock  := {| | 100             }
         ::firstPosBlock := {| | 0               }
         ::drawMode      := XBP_DRAW_OWNER

      ::lDescend:= .F.
   RETURN self

   INLINE METHOD SetColOrder( oCol )
   *********************************
      LOCAL _oCol

      IF oCol # ::oColOrder
         ::lDescend:= .F.
         IF ::oColOrder # nil
            _oCol:= ::oColOrder
            ::oColOrder:= oCol
            _oCol:heading:invalidateRect()
         ELSE
            ::oColOrder:= oCol
         ENDIF
         ( ::nArea )->( OrdSetFocus( AScan( ::aColumns, oCol ) ) )
      ELSE
         ::lDescend:= ! ::lDescend
         ( ::nArea )->( OrdSetFocus( IIF( ::lDescend, 4, 0 ) + AScan( ::aColumns, oCol ) ) )
      ENDIF
      oCol:heading:invalidateRect()
      ::goTop()
      ::refreshAll()
   RETURN self

   INLINE METHOD CustomDrawCell( oPS, aInfo )
   ******************************************
      IF  aInfo[ XBP_DRAWINFO_COLUMN ] == ::oColOrder .AND. aInfo[ XBP_DRAWINFO_AREA ] = aInfo[ XBP_DRAWINFO_COLUMN ]:heading
         oPS:setColor( GRA_CLR_DARKGRAY )
         ::drawMarker( oPS, { aInfo[ XBP_DRAWINFO_RECT, 3 ] - 8, aInfo[ XBP_DRAWINFO_RECT, 2 ] + 8 } )
      ENDIF
   RETURN .T.
ENDCLASS

*******************************
CLASS XbpColTest FROM XbpColumn
*******************************
EXPORTED:
   INLINE METHOD Init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ************************************************************************
      ::XbpColumn:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
      ::heading := XbpHeadingTest():new( self, self )
      ::drawMode:= XBP_DRAW_OWNER
   RETURN self
ENDCLASS

**************************************
CLASS XbpHeadingTest FROM XbpCellGroup
**************************************
EXPORTED:
   VAR nDefaultPointer

   INLINE METHOD Destroy ; ::XbpCellGroup:destroy() ; ::nDefaultPointer:= nil ; RETURN self
   *********************

   INLINE METHOD create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   **************************************************************************
      ::XbpCellGroup:MaxRow := 2
      ::XbpCellGroup:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
      ::SetCell(2,SubStr(::referenceString,7,3))
      ::nDefaultPointer:= DEFAULT_HEADING_POINTER
      ::setPointer( nil, ::nDefaultPointer, XBPWINDOW_POINTERTYPE_SYSPOINTER )
   RETURN self

   INLINE METHOD SetPointer( cDllName, nResID, nType )
   ***************************************************
   RETURN ::XbpCellGroup:setPointer( cDllName, IIF( nResID = XBPSTATIC_SYSICON_DEFAULT .OR. nResID = XBPSTATIC_SYSICON_ARROW, ;
                                                    ::nDefaultPointer, nResID ), nType )
   INLINE METHOD lbClick
   *********************
      LOCAL oCol:= ::setParent()
      oCol:setParent():setParent():setParent():setColOrder( oCol )
   RETURN self
ENDCLASS



